@@ -36,13 +36,15 @@ String NodeValue::value () const {
 namespace {
 
 struct EvalState {
-    EvalState (EvalState *p)
-     : root (NULL), attr (NULL), process_list (NULL), parent (p),
+    EvalState (EvalState *p, const String &root_tag=String())
+     : def_root_tag (root_tag), root (NULL), attr (NULL),
+       process_list (NULL), parent (p),
        sequence (1), ref_count (0) {}
 
     void addRef () { ++ref_count; }
     void removeRef () { if (--ref_count == 0) delete this; }
 
+    String def_root_tag;
     Node *root;
     Attribute *attr;
     NodeValueList *process_list;
@@ -250,6 +252,12 @@ struct CurrentDate : public StringBase {
     virtual String toString () const;
 };
 
+struct Sort : public AST {
+    Sort (EvalState *ev) : AST (ev) {}
+
+    virtual NodeValueList *toNodeList () const;
+};
+
 struct Multiply : public NumberBase {
     Multiply (EvalState *ev, AST *children) : NumberBase (ev) {
         first_child = children;
@@ -749,6 +757,66 @@ String CurrentDate::toString () const {
     return string;
 }
 
+static void sortList (NodeValueList *lst, Expression *expr) {
+    NodeValueItem *cur = lst->first ();
+    NodeValueList lt;
+    NodeValueList gt;
+    expr->setRoot (cur->data.node);
+    String str = expr->toString ();
+    for (NodeValueItem *itm = cur->nextSibling (); itm; ) {
+        NodeValueItem *next = itm->nextSibling ();
+        expr->setRoot (itm->data.node);
+        int cmp = str.compare (expr->toString ());
+        if (cmp < 0) {
+            NodeValueItemPtr s = itm;
+            lst->remove (itm);
+            gt.append (itm);
+        } else if (cmp > 0) {
+            NodeValueItemPtr s = itm;
+            lst->remove (itm);
+            lt.append (itm);
+        }
+        itm = next;
+    }
+    if (lt.first ()) {
+        sortList (&lt, expr);
+        for (NodeValueItem *itm = lt.first (); itm; itm = lt.first ()) {
+            NodeValueItemPtr s = itm;
+            lt.remove (itm);
+            lst->insertBefore (itm, cur);
+        }
+    }
+    if (gt.first ()) {
+        sortList (&gt, expr);
+        for (NodeValueItem *itm = gt.first (); itm; itm = gt.first ()) {
+            NodeValueItemPtr s = itm;
+            gt.remove (itm);
+            lst->append (itm);
+        }
+    }
+}
+
+NodeValueList *Sort::toNodeList () const {
+    if (first_child) {
+        Expression *exp = evaluateExpr (first_child->toString ());
+        if (exp) {
+            exp->setRoot (eval_state->root);
+            NodeValueList *lst = exp->toNodeList ();
+            if (lst->first () && first_child->next_sibling) {
+                Expression *sort_exp =
+                    evaluateExpr (first_child->next_sibling->toString ());
+                if (sort_exp) {
+                    sortList (lst, sort_exp);
+                    delete sort_exp;
+                }
+            }
+            delete exp;
+            return lst;
+        }
+    }
+    return AST::toNodeList ();
+}
+
 #define BIN_OP_TO_INT(NAME,OP)                                           \
     AST *second_child = first_child->next_sibling;                       \
     AST::Type t1 = first_child->type ();                                 \
@@ -1080,8 +1148,10 @@ static bool parseIdentifier (const char *str, const char **end, AST *ast) {
         return false;
     if (*str == '/')
         ++str;
-    else if (!ast->eval_state->parent)
-        appendASTChild (&ident, new Step (ast->eval_state, "data", NULL));
+    else if (!ast->eval_state->parent &&
+            !ast->eval_state->def_root_tag.isEmpty ())
+        appendASTChild (&ident, new Step (ast->eval_state,
+                    (const char *) ast->eval_state->def_root_tag, NULL));
     while (parseStep (str, end, &ident)) {
         str = *end;
         has_any = true;
@@ -1175,6 +1245,8 @@ static bool parseFunction (const char *str, const char **end, AST *ast) {
                     func = new Number (ast->eval_state);
                 else if (name == "position")
                     func = new Position (ast->eval_state);
+                else if (name == "sort")
+                    func = new Sort (ast->eval_state);
                 else
                     return false;
                 appendASTChild (ast, func);
@@ -1371,8 +1443,8 @@ static bool parseStatement (const char *str, const char **end, AST *ast) {
     return false;
 }
 
-Expression *KMPlayer::evaluateExpr (const String &expr) {
-    EvalState *eval_state = new EvalState (NULL);
+Expression *KMPlayer::evaluateExpr (const String &expr, const String &root) {
+    EvalState *eval_state = new EvalState (NULL, root);
     AST ast (eval_state);
     const char *end;
     if (parseStatement ((const char *)expr, &end, &ast)) {
