@@ -23,6 +23,7 @@
 #include <unistd.h>
 //#include <errno.h>
 #include <string.h>
+#include <map>
 
 #include <gtk/gtk.h>
 
@@ -197,11 +198,12 @@ void Playlist::show (View *view)
         if (!view->before)
             before = meta->nextSibling ();
 
-        GHashTable *groups = NULL;
+        typedef std::map <String, PlaylistGroup *> GroupMap;
+        GroupMap *groups = NULL;
         String groupBy = view->getAttribute ("groupBy");
         TrieString attr_groupBy (groupBy);
         if (!groupBy.isEmpty ())
-            groups = g_hash_table_new (g_str_hash, g_str_equal);
+            groups = new GroupMap;
 
         view->show->setRoot (this);
         NodeValueList *lst = view->show->toNodeList ();
@@ -220,24 +222,28 @@ void Playlist::show (View *view)
 
                 groupBy = e->getAttribute (attr_groupBy).stripWhiteSpace ();
                 if (groups && !groupBy.isEmpty ()) {
-                    Node *group = (Node *)g_hash_table_lookup (groups, (const char *)groupBy);
-                    if (!group) {
+                    GroupMap::iterator it = groups->find (groupBy);
+                    PlaylistGroup *group;
+                    if (it == groups->end ()) {
                         PlaylistGroup *g = new PlaylistGroup(m_doc, app, false);
-                        insertBefore (g, before);
                         g->setAttribute (StringPool::attr_title, groupBy);
                         g->setAuxiliaryNode (true);
                         g->closed ();
                         group = g;
-                        g_hash_table_insert (groups,
-                                (void *) (const char *) groupBy, (Node *) g);
+                        it = groups->insert (std::make_pair (groupBy, g)).first;
+                        Node *b = before;
+                        if (++it != groups->end ())
+                            b = it->second;
+                        insertBefore (g, b);
+                    } else {
+                        group = it->second;
                     }
                     group->appendChild (itm);
                 } else {
                     insertBefore (itm, before);
                 }
             }
-        if (groups)
-            g_hash_table_destroy (groups);
+        delete groups;
         delete lst;
         debugLog () << "Playlist;;show open " << open << endl;
         if (!open)
@@ -396,9 +402,10 @@ void *PlaylistGroupBase::role (KMPlayer::RoleType msg, void *content)
     return Element::role (msg, content);
 }
 
-PlaylistGroup::PlaylistGroup (NodePtr &doc, Application *a, bool lm)
- : PlaylistGroupBase (doc, KMPlayer::id_node_group_node, a, lm)
-{}
+const char *PlaylistGroup::nodeName () const
+{
+    return "group";
+}
 
 PlaylistMetaGroup::PlaylistMetaGroup (NodePtr &doc, Application *a)
  : PlaylistGroupBase (doc, KMPlayer::id_node_meta_group, a, false)
