@@ -1707,6 +1707,7 @@ inline SimpleSAXParser::~SimpleSAXParser () {};
 KMPLAYER_EXPORT
 void KMPlayer::readXML (NodePtr root, TextStream & in, const String & firstline, bool set_opener) {
     DocumentBuilder builder (root, set_opener);
+    root->opened ();
     SimpleSAXParser parser (builder);
     if (!firstline.isEmpty ()) {
         String str (firstline + char_nl);
@@ -1715,6 +1716,8 @@ void KMPlayer::readXML (NodePtr root, TextStream & in, const String & firstline,
     }
     if (!in.atEnd ())
         parser.parse (in);
+    if (root->open) // endTag may have closed it
+        root->closed ();
     for (NodePtr e = root->parentNode (); e; e = e->parentNode ()) {
         if (e->open)
             break;
