@@ -231,6 +231,7 @@ enum MessageType
     MsgMediaPrefetch,
     MsgMediaReady,
     MsgMediaUpdated,
+    MsgListUpdated,
     MsgEventPostponed,
     MsgSurfaceBoundsUpdate,      // bool parent surface resized
     MsgSurfaceUpdate,
@@ -525,11 +526,8 @@ const short id_node_download = 4;
 const short id_node_text = 5;
 const short id_node_cdata = 6;
 
-const short id_node_group_node = 25;
-const short id_node_playlist_document = 26;
-const short id_node_playlist_item = 27;
-const short id_node_param = 28;
-const short id_node_svg = 29;
+const short id_node_param = 15;
+const short id_node_svg = 16;
 
 /*
  * Element node, XML node that can have attributes
