@@ -767,7 +767,7 @@ GtkWidget * createTreeView (Application * app) {
     config_pix = gtk_icon_theme_load_icon (icon_theme,
             "general_settings", 26, (GtkIconLookupFlags)0, 0L);
     view_pix = gtk_icon_theme_load_icon (icon_theme,
-            "qgn_widg_mplayer_play", 26, (GtkIconLookupFlags)0, 0L);
+            "general_presence_cultural_activities", 26, (GtkIconLookupFlags)0, 0L);
     music_pix = gtk_icon_theme_load_icon (icon_theme,
             "general_audio_file", 26, (GtkIconLookupFlags)0, 0L);
     video_pix = gtk_icon_theme_load_icon (icon_theme,
